@@ -1,8 +1,8 @@
+#[allow(deprecated)]
+use solana_sdk::sysvar::recent_blockhashes;
 use {
     serde::{Deserialize, Serialize},
-    solana_sdk::{
-        fee_calculator::FeeCalculator, hash::Hash, sysvar::recent_blockhashes, timing::timestamp,
-    },
+    solana_sdk::{fee_calculator::FeeCalculator, hash::Hash, timing::timestamp},
     std::collections::HashMap,
 };
 
@@ -48,8 +48,10 @@ impl BlockhashQueue {
         self.last_hash.expect("no hash has been set")
     }
 
-    pub fn get_fee_calculator(&self, hash: &Hash) -> Option<&FeeCalculator> {
-        self.ages.get(hash).map(|hash_age| &hash_age.fee_calculator)
+    pub fn get_lamports_per_signature(&self, hash: &Hash) -> Option<u64> {
+        self.ages
+            .get(hash)
+            .map(|hash_age| hash_age.fee_calculator.lamports_per_signature)
     }
 
     /// Check if the age of the hash is within the max_age
@@ -68,16 +70,15 @@ impl BlockhashQueue {
     }
 
     /// check if hash is valid
-    #[cfg(test)]
-    pub fn check_hash(&self, hash: Hash) -> bool {
-        self.ages.get(&hash).is_some()
+    pub fn check_hash(&self, hash: &Hash) -> bool {
+        self.ages.get(hash).is_some()
     }
 
-    pub fn genesis_hash(&mut self, hash: &Hash, fee_calculator: &FeeCalculator) {
+    pub fn genesis_hash(&mut self, hash: &Hash, lamports_per_signature: u64) {
         self.ages.insert(
             *hash,
             HashAge {
-                fee_calculator: fee_calculator.clone(),
+                fee_calculator: FeeCalculator::new(lamports_per_signature),
                 hash_height: 0,
                 timestamp: timestamp(),
             },
@@ -90,7 +91,7 @@ impl BlockhashQueue {
         hash_height - age.hash_height <= max_age as u64
     }
 
-    pub fn register_hash(&mut self, hash: &Hash, fee_calculator: &FeeCalculator) {
+    pub fn register_hash(&mut self, hash: &Hash, lamports_per_signature: u64) {
         self.hash_height += 1;
         let hash_height = self.hash_height;
 
@@ -104,7 +105,7 @@ impl BlockhashQueue {
         self.ages.insert(
             *hash,
             HashAge {
-                fee_calculator: fee_calculator.clone(),
+                fee_calculator: FeeCalculator::new(lamports_per_signature),
                 hash_height,
                 timestamp: timestamp(),
             },
@@ -123,10 +124,15 @@ impl BlockhashQueue {
         None
     }
 
+    #[deprecated(
+        since = "1.9.0",
+        note = "Please do not use, will no longer be available in the future"
+    )]
+    #[allow(deprecated)]
     pub fn get_recent_blockhashes(&self) -> impl Iterator<Item = recent_blockhashes::IterItem> {
-        (&self.ages)
-            .iter()
-            .map(|(k, v)| recent_blockhashes::IterItem(v.hash_height, k, &v.fee_calculator))
+        (&self.ages).iter().map(|(k, v)| {
+            recent_blockhashes::IterItem(v.hash_height, k, v.fee_calculator.lamports_per_signature)
+        })
     }
 
     pub(crate) fn len(&self) -> usize {
@@ -135,21 +141,21 @@ impl BlockhashQueue {
 }
 #[cfg(test)]
 mod tests {
+    #[allow(deprecated)]
+    use solana_sdk::sysvar::recent_blockhashes::IterItem;
     use {
         super::*,
         bincode::serialize,
-        solana_sdk::{
-            clock::MAX_RECENT_BLOCKHASHES, hash::hash, sysvar::recent_blockhashes::IterItem,
-        },
+        solana_sdk::{clock::MAX_RECENT_BLOCKHASHES, hash::hash},
     };
 
     #[test]
     fn test_register_hash() {
         let last_hash = Hash::default();
         let mut hash_queue = BlockhashQueue::new(100);
-        assert!(!hash_queue.check_hash(last_hash));
-        hash_queue.register_hash(&last_hash, &FeeCalculator::default());
-        assert!(hash_queue.check_hash(last_hash));
+        assert!(!hash_queue.check_hash(&last_hash));
+        hash_queue.register_hash(&last_hash, 0);
+        assert!(hash_queue.check_hash(&last_hash));
         assert_eq!(hash_queue.hash_height(), 1);
     }
 
@@ -159,15 +165,15 @@ mod tests {
         let last_hash = hash(&serialize(&0).unwrap());
         for i in 0..102 {
             let last_hash = hash(&serialize(&i).unwrap());
-            hash_queue.register_hash(&last_hash, &FeeCalculator::default());
+            hash_queue.register_hash(&last_hash, 0);
         }
         // Assert we're no longer able to use the oldest hash.
-        assert!(!hash_queue.check_hash(last_hash));
+        assert!(!hash_queue.check_hash(&last_hash));
         assert_eq!(None, hash_queue.check_hash_age(&last_hash, 0));
 
         // Assert we are not able to use the oldest remaining hash.
         let last_valid_hash = hash(&serialize(&1).unwrap());
-        assert!(hash_queue.check_hash(last_valid_hash));
+        assert!(hash_queue.check_hash(&last_valid_hash));
         assert_eq!(Some(false), hash_queue.check_hash_age(&last_valid_hash, 0));
     }
 
@@ -176,7 +182,7 @@ mod tests {
     fn test_queue_init_blockhash() {
         let last_hash = Hash::default();
         let mut hash_queue = BlockhashQueue::new(100);
-        hash_queue.register_hash(&last_hash, &FeeCalculator::default());
+        hash_queue.register_hash(&last_hash, 0);
         assert_eq!(last_hash, hash_queue.last_hash());
         assert_eq!(Some(true), hash_queue.check_hash_age(&last_hash, 0));
     }
@@ -184,16 +190,19 @@ mod tests {
     #[test]
     fn test_get_recent_blockhashes() {
         let mut blockhash_queue = BlockhashQueue::new(MAX_RECENT_BLOCKHASHES);
+        #[allow(deprecated)]
         let recent_blockhashes = blockhash_queue.get_recent_blockhashes();
         // Sanity-check an empty BlockhashQueue
         assert_eq!(recent_blockhashes.count(), 0);
         for i in 0..MAX_RECENT_BLOCKHASHES {
             let hash = hash(&serialize(&i).unwrap());
-            blockhash_queue.register_hash(&hash, &FeeCalculator::default());
+            blockhash_queue.register_hash(&hash, 0);
         }
+        #[allow(deprecated)]
         let recent_blockhashes = blockhash_queue.get_recent_blockhashes();
         // Verify that the returned hashes are most recent
-        for IterItem(_slot, hash, _fee_calc) in recent_blockhashes {
+        #[allow(deprecated)]
+        for IterItem(_slot, hash, _lamports_per_signature) in recent_blockhashes {
             assert_eq!(
                 Some(true),
                 blockhash_queue.check_hash_age(hash, MAX_RECENT_BLOCKHASHES)
