@@ -1,11 +1,9 @@
-#[cfg(RUSTC_WITH_SPECIALIZATION)]
-use solana_frozen_abi::abi_example::IgnoreAsHelper;
 use {
     crate::{
         accounts::Accounts,
         accounts_db::{
             AccountShrinkThreshold, AccountStorageEntry, AccountsDb, AccountsDbConfig, AppendVecId,
-            BankHashInfo, IndexGenerationInfo,
+            BankHashInfo, IndexGenerationInfo, SnapshotStorage,
         },
         accounts_index::AccountSecondaryIndexes,
         accounts_update_notifier_interface::AccountsUpdateNotifier,
@@ -16,7 +14,6 @@ use {
         epoch_stakes::EpochStakes,
         hardened_unpack::UnpackedAppendVecMap,
         rent_collector::RentCollector,
-        serde_snapshot::future::SerializableStorage,
         stakes::Stakes,
     },
     bincode::{self, config::Options, Error},
@@ -45,20 +42,17 @@ use {
         },
         thread::Builder,
     },
+    storage::SerializableStorage,
 };
 
-mod common;
-mod future;
+mod newer;
+mod storage;
 mod tests;
 mod utils;
 
 // a number of test cases in accounts_db use this
 #[cfg(test)]
-pub(crate) use self::tests::reconstruct_accounts_db_via_serialization;
-pub(crate) use crate::accounts_db::{SnapshotStorage, SnapshotStorages};
-use future::Context as TypeContextFuture;
-#[allow(unused_imports)]
-use utils::{serialize_iter_as_map, serialize_iter_as_seq, serialize_iter_as_tuple};
+pub(crate) use tests::reconstruct_accounts_db_via_serialization;
 
 #[derive(Copy, Clone, Eq, PartialEq)]
 pub(crate) enum SerdeStyle {
@@ -204,15 +198,15 @@ where
     R: Read,
 {
     macro_rules! INTO {
-        ($x:ident) => {{
+        ($style:ident) => {{
             let (full_snapshot_bank_fields, full_snapshot_accounts_db_fields) =
-                $x::deserialize_bank_fields(snapshot_streams.full_snapshot_stream)?;
+                $style::Context::deserialize_bank_fields(snapshot_streams.full_snapshot_stream)?;
             let (incremental_snapshot_bank_fields, incremental_snapshot_accounts_db_fields) =
                 if let Some(ref mut incremental_snapshot_stream) =
                     snapshot_streams.incremental_snapshot_stream
                 {
                     let (bank_fields, accounts_db_fields) =
-                        $x::deserialize_bank_fields(incremental_snapshot_stream)?;
+                        $style::Context::deserialize_bank_fields(incremental_snapshot_stream)?;
                     (Some(bank_fields), Some(accounts_db_fields))
                 } else {
                     (None, None)
@@ -242,7 +236,7 @@ where
         }};
     }
     match serde_style {
-        SerdeStyle::Newer => INTO!(TypeContextFuture),
+        SerdeStyle::Newer => INTO!(newer),
     }
     .map_err(|err| {
         warn!("bankrc_from_stream error: {:?}", err);
@@ -260,10 +254,10 @@ where
     W: Write,
 {
     macro_rules! INTO {
-        ($x:ident) => {
+        ($style:ident) => {
             bincode::serialize_into(
                 stream,
-                &SerializableBankAndStorage::<$x> {
+                &SerializableBankAndStorage::<$style::Context> {
                     bank,
                     snapshot_storages,
                     phantom: std::marker::PhantomData::default(),
@@ -272,7 +266,7 @@ where
         };
     }
     match serde_style {
-        SerdeStyle::Newer => INTO!(TypeContextFuture),
+        SerdeStyle::Newer => INTO!(newer),
     }
     .map_err(|err| {
         warn!("bankrc_to_stream error: {:?}", err);
@@ -312,7 +306,7 @@ impl<'a, C: TypeContext<'a>> Serialize for SerializableAccountsDb<'a, C> {
 }
 
 #[cfg(RUSTC_WITH_SPECIALIZATION)]
-impl<'a, C> IgnoreAsHelper for SerializableAccountsDb<'a, C> {}
+impl<'a, C> solana_frozen_abi::abi_example::IgnoreAsHelper for SerializableAccountsDb<'a, C> {}
 
 #[allow(clippy::too_many_arguments)]
 fn reconstruct_bank_from_fields<E>(
